import * as React from 'react';
import { delegateCombine, delegateRemove } from "igniteui-react-core";
import { IgrToolActionCollection } from "./igr-tool-action-collection";
import { Visibility_$type } from "igniteui-react-core";
import { IgrToolCommandEventArgs } from "./igr-tool-command-event-args";
import { IgrToolActionPerformedEventArgs } from "./igr-tool-action-performed-event-args";
import { ToolAction } from "./ToolAction";
import { TypeRegistrar } from "igniteui-react-core";
import { NamePatcher, getModifiedProps, isValidProp, brushToString, stringToBrush, ensureBool, ensureEnum, toSpinal, initializePropertiesFromCss } from "igniteui-react-core";
import { ToolActionCollection as ToolActionCollection_internal } from "./ToolActionCollection";
import { SyncableObservableCollection$1 } from "igniteui-react-core";
import { FontInfo } from "igniteui-react-core";
/**
 * Represents base class for all tool actions in toolbar
*/
export class IgrToolAction extends React.Component {
    createImplementation() {
        return null;
    }
    /**
     * @hidden
     */
    get i() {
        return this._implementation;
    } /**
                                 * @hidden
                                 */
    static _createFromInternal(internal) {
        if (!internal) {
            return null;
        }
        if (!internal.$type) {
            return null;
        }
        let name = internal.$type.name;
        let externalName = "Igr" + name;
        if (!TypeRegistrar.isRegistered(externalName)) {
            return null;
        }
        return TypeRegistrar.create(externalName);
    }
    onImplementationCreated() {
    }
    constructor(props) {
        super(props);
        this._actions = null;
        this.__p = null;
        this._hasUserValues = new Set();
        this._stylingContainer = null;
        this._stylingParent = null;
        this._inStyling = false;
        this._onCommand = null;
        this._onCommand_wrapped = null;
        this._performed = null;
        this._performed_wrapped = null;
        if (this._styling) {
            NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
        }
        this._implementation = this.createImplementation();
        this._implementation.externalObject = this;
        this.onImplementationCreated();
        if (this._initializeAdapters) {
            this._initializeAdapters();
        }
    }
    componentDidMount() {
        for (const p of Object.keys(this.props)) {
            if (isValidProp(this, p)) {
                this[p] = this.props[p];
            }
        }
    }
    shouldComponentUpdate(nextProps, nextState) {
        const mod = getModifiedProps(this.props, nextProps);
        for (const p of Object.keys(mod)) {
            if (isValidProp(this, p)) {
                this[p] = mod[p];
            }
        }
        return true;
    }
    render() {
        return null;
    }
    get id() {
        return this.i.cw;
    }
    /**
     * Gets the sub actions associated with this tool action.
    */
    get actions() {
        if (this._actions === null) {
            let coll = new IgrToolActionCollection();
            let innerColl = this.i.actions;
            if (!innerColl) {
                innerColl = new ToolActionCollection_internal();
            }
            this._actions = coll._fromInner(innerColl);
        }
        return this._actions;
    }
    set actions(v) {
        if (this._actions !== null) {
            this._actions._setSyncTarget(null);
            this._actions = null;
        }
        let coll = new IgrToolActionCollection();
        this._actions = coll._fromOuter(v);
        let syncColl = new SyncableObservableCollection$1(ToolAction.$type);
        let innerColl = this.i.actions;
        if (!innerColl) {
            innerColl = new ToolActionCollection_internal();
        }
        syncColl._inner = innerColl;
        syncColl.clear();
        this._actions._setSyncTarget(syncColl);
    }
    /**
     * Gets or sets the width to use for this tool action.
    */
    get width() {
        return this.i.bl;
    }
    set width(v) {
        this.i.bl = +v;
    }
    get parentId() {
        return this.i.c5;
    }
    set parentId(v) {
        this.i.c5 = v;
    }
    /**
     * Gets or sets the Id of the action that this action will be inserted before.
     * Subsequent actions will continue to insert before the target action, following
     * the original action that set this property. If using this property do not use the
     * AfterId property.
     * This property is only relevant during initial insertion into the toolbar. Changing the property
     * after the action has already been added will have no effect.
    */
    get beforeId() {
        return this.i.cc;
    }
    set beforeId(v) {
        this.i.cc = v;
    }
    /**
     * Gets or sets the Id of the action that this action will be inserted after.
     * Subsequent actions will continue to insert after the target action, following
     * the original action that set this property. If using this property do not use the
     * BeforeId property.
     * This property is only relevant during initial insertion into the toolbar. Changing the property
     * after the action has already been added will have no effect.
    */
    get afterId() {
        return this.i.b9;
    }
    set afterId(v) {
        this.i.b9 = v;
    }
    /**
     * Sets to the Id of the action that this action will overlay itself on to.
     * The purpose of this property is to allow users to override actions automatically generated by
     * toolbar targets such as the DataChart. These generated actions all have unique Ids associated
     * with them that can be used for the OverlayId.
     * These overlaid actions will
     * not
     * replace the generated one but will have its properties
     * overlaid onto the generated actions.
    */
    get overlayId() {
        return this.i.c0;
    }
    set overlayId(v) {
        this.i.c0 = v;
    }
    get commandId() {
        return this.i.ch;
    }
    set commandId(v) {
        this.i.ch = v;
    }
    get commandArgument() {
        return this.i.ce;
    }
    set commandArgument(v) {
        this.i.ce = v;
    }
    get commandArgumentValue() {
        return this.i.bn;
    }
    set commandArgumentValue(v) {
        this.i.bn = v;
    }
    get paddingLeft() {
        return this.i.bh;
    }
    set paddingLeft(v) {
        this.i.bh = +v;
    }
    get actualPaddingLeft() {
        return this.i.a5;
    }
    set actualPaddingLeft(v) {
        this.i.a5 = +v;
    }
    get paddingRight() {
        return this.i.bi;
    }
    set paddingRight(v) {
        this.i.bi = +v;
    }
    get actualPaddingRight() {
        return this.i.a6;
    }
    set actualPaddingRight(v) {
        this.i.a6 = +v;
    }
    get paddingBottom() {
        return this.i.bg;
    }
    set paddingBottom(v) {
        this.i.bg = +v;
    }
    get actualPaddingBottom() {
        return this.i.a4;
    }
    set actualPaddingBottom(v) {
        this.i.a4 = +v;
    }
    get paddingTop() {
        return this.i.bj;
    }
    set paddingTop(v) {
        this.i.bj = +v;
    }
    get actualPaddingTop() {
        return this.i.a7;
    }
    set actualPaddingTop(v) {
        this.i.a7 = +v;
    }
    /**
     * Gets the actual hover background used by the tool action.
    */
    get actualHoverBackground() {
        return brushToString(this.i.eu);
    }
    set actualHoverBackground(v) {
        this.i.eu = stringToBrush(v);
    }
    /**
     * Gets or sets the hover background for this tool action.
    */
    get hoverBackground() {
        return brushToString(this.i.e0);
    }
    set hoverBackground(v) {
        this.i.e0 = stringToBrush(v);
    }
    /**
     * Gets the actual background used by the tool action.
    */
    get actualBackground() {
        return brushToString(this.i.es);
    }
    set actualBackground(v) {
        this.i.es = stringToBrush(v);
    }
    /**
     * Gets or sets the background color of the tool action.
    */
    get background() {
        return brushToString(this.i.ey);
    }
    set background(v) {
        this.i.ey = stringToBrush(v);
    }
    /**
     * Gets the actual label text color.
    */
    get actualTextColor() {
        return brushToString(this.i.ex);
    }
    set actualTextColor(v) {
        this.i.ex = stringToBrush(v);
    }
    /**
     * Gets or sets the label text color.
    */
    get textColor() {
        return brushToString(this.i.e3);
    }
    set textColor(v) {
        this.i.e3 = stringToBrush(v);
    }
    /**
     * Gets the actual label text color when the tool is disabled.
    */
    get actualDisabledTextColor() {
        return brushToString(this.i.et);
    }
    set actualDisabledTextColor(v) {
        this.i.et = stringToBrush(v);
    }
    /**
     * Gets or sets the label text color when the tool is disabled.
    */
    get disabledTextColor() {
        return brushToString(this.i.ez);
    }
    set disabledTextColor(v) {
        this.i.ez = stringToBrush(v);
    }
    /**
     * Gets the actual font used.
    */
    get actualTextStyle() {
        if (this.i.r == null) {
            return null;
        }
        return this.i.r.fontString;
    }
    set actualTextStyle(v) {
        let fi = new FontInfo();
        fi.fontString = v;
        this.i.r = fi;
    }
    /**
     * Gets or sets the font to use.
    */
    get textStyle() {
        if (this.i.t == null) {
            return null;
        }
        return this.i.t.fontString;
    }
    set textStyle(v) {
        let fi = new FontInfo();
        fi.fontString = v;
        this.i.t = fi;
    }
    get isDisabled() {
        return this.i.aj;
    }
    set isDisabled(v) {
        this.i.aj = ensureBool(v);
    }
    get name() {
        return this.i.cy;
    }
    set name(v) {
        this.i.cy = v;
    }
    /**
     * Gets or sets the title of the tool.
    */
    get title() {
        return this.i.de;
    }
    set title(v) {
        this.i.de = v;
    }
    /**
     * Gets or sets the subtitle of the tool.
    */
    get subtitle() {
        return this.i.da;
    }
    set subtitle(v) {
        this.i.da = v;
    }
    get iconName() {
        return this.i.cp;
    }
    set iconName(v) {
        this.i.cp = v;
    }
    get iconCollectionName() {
        return this.i.cm;
    }
    set iconCollectionName(v) {
        this.i.cm = v;
    }
    /**
     * Gets or sets the icon viewbox left.
    */
    get iconViewBoxLeft() {
        return this.i.bd;
    }
    set iconViewBoxLeft(v) {
        this.i.bd = +v;
    }
    /**
     * Gets or sets the icon viewbox top.
    */
    get iconViewBoxTop() {
        return this.i.be;
    }
    set iconViewBoxTop(v) {
        this.i.be = +v;
    }
    /**
     * Gets or sets the icon viewbox width.
    */
    get iconViewBoxWidth() {
        return this.i.bf;
    }
    set iconViewBoxWidth(v) {
        this.i.bf = +v;
    }
    /**
     * Gets or sets the icon viewbox height.
    */
    get iconViewBoxHeight() {
        return this.i.bc;
    }
    set iconViewBoxHeight(v) {
        this.i.bc = +v;
    }
    /**
     * Gets or sets the icon fill color.
    */
    get iconFill() {
        return brushToString(this.i.e1);
    }
    set iconFill(v) {
        this.i.e1 = stringToBrush(v);
    }
    /**
     * Gets the actual icon fill color.
    */
    get actualIconFill() {
        return brushToString(this.i.ev);
    }
    set actualIconFill(v) {
        this.i.ev = stringToBrush(v);
    }
    /**
     * Gets or sets the icon stroke color.
    */
    get iconStroke() {
        return brushToString(this.i.e2);
    }
    set iconStroke(v) {
        this.i.e2 = stringToBrush(v);
    }
    /**
     * Gets the actual icon stroke color.
    */
    get actualIconStroke() {
        return brushToString(this.i.ew);
    }
    set actualIconStroke(v) {
        this.i.ew = stringToBrush(v);
    }
    /**
     * Gets or sets the tool action visibility.
    */
    get visibility() {
        return this.i.e5;
    }
    set visibility(v) {
        this.i.e5 = ensureEnum(Visibility_$type, v);
    }
    /**
     * Gets or sets if the tool action will close its parent dropdown when executed by the user.
    */
    get closeOnExecute() {
        return this.i.ae;
    }
    set closeOnExecute(v) {
        this.i.ae = ensureBool(v);
    }
    findByName(name) {
        if (this.findEphemera) {
            if (name && name.indexOf("@@e:") == 0) {
                return this.findEphemera(name);
            }
        }
        if (this.actions != null && this.actions.findByName && this.actions.findByName(name)) {
            return this.actions.findByName(name);
        }
        return null;
    }
    get hasUserValues() {
        return this._hasUserValues;
    }
    __m(propertyName) {
        if (!this._inStyling) {
            this._hasUserValues.add(propertyName);
        }
    }
    _styling(container, component, parent) {
        if (this._inStyling) {
            return;
        }
        this._inStyling = true;
        this._stylingContainer = container;
        this._stylingParent = component;
        let genericPrefix = "";
        let typeName = this.i.$type.name;
        if (typeName.indexOf("Xam") === 0) {
            typeName = typeName.substring(3);
        }
        genericPrefix = toSpinal("ToolAction");
        let additionalPrefixes = [];
        let prefix = toSpinal(typeName);
        additionalPrefixes.push(prefix + "-");
        let b = this.i.$type.baseType;
        while (b && b.name != "Object" &&
            b.name != "Base" &&
            b.name != "Control" &&
            b.Name != "DependencyObject" &&
            b.Name != "FrameworkElement") {
            typeName = b.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            let basePrefix = toSpinal(typeName);
            additionalPrefixes.push(basePrefix + "-");
            b = b.baseType;
        }
        if (parent) {
            let parentTypeName = parent.i.$type.name;
            if (parentTypeName.indexOf("Xam") === 0) {
                parentTypeName = parentTypeName.substring(3);
            }
            let parentPrefix = toSpinal(parentTypeName);
            additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
            additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
        }
        initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
        if (this._otherStyling) {
            this._otherStyling(container, component, parent);
        }
        this._inStyling = false;
    }
    cleanUp() {
        this.i.dh();
    }
    fromInfo(info) {
        this.i.dt(info);
    }
    openSubMenu() {
        this.i.d6();
    }
    closeSubmenu() {
        this.i.dj();
    }
    get onCommand() {
        return this._onCommand;
    }
    set onCommand(ev) {
        if (this._onCommand_wrapped !== null) {
            this.i.onCommand = delegateRemove(this.i.onCommand, this._onCommand_wrapped);
            this._onCommand_wrapped = null;
            this._onCommand = null;
        }
        this._onCommand = ev;
        this._onCommand_wrapped = (o, e) => {
            let outerArgs = new IgrToolCommandEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeOnCommand) {
                this.beforeOnCommand(this, outerArgs);
            }
            if (this._onCommand) {
                this._onCommand(this, outerArgs);
            }
        };
        this.i.onCommand = delegateCombine(this.i.onCommand, this._onCommand_wrapped);
        ;
    }
    get performed() {
        return this._performed;
    }
    set performed(ev) {
        if (this._performed_wrapped !== null) {
            this.i.performed = delegateRemove(this.i.performed, this._performed_wrapped);
            this._performed_wrapped = null;
            this._performed = null;
        }
        this._performed = ev;
        this._performed_wrapped = (o, e) => {
            let outerArgs = new IgrToolActionPerformedEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforePerformed) {
                this.beforePerformed(this, outerArgs);
            }
            if (this._performed) {
                this._performed(this, outerArgs);
            }
        };
        this.i.performed = delegateCombine(this.i.performed, this._performed_wrapped);
        ;
    }
}
